@@ -455,7 +455,7 @@ static int virLockManagerSanlockNew(virLockManagerPtr lock,
     size_t i;
     int resCount = 0;
 
-    virCheckFlags(0, -1);
+    virCheckFlags(VIR_LOCK_MANAGER_NEW_STARTED, -1);
 
     if (!driver) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
@@ -497,8 +497,11 @@ static int virLockManagerSanlockNew(virLockManagerPtr lock,
      * if it returns any other error (rv < 0), then we cannot fail due
      * to back-compat.  So this whole call is non-fatal, because it's
      * called from all over the place (it will usually fail).  It merely
-     * updates privateData. */
-    if (sanlock_inquire(-1, priv->vm_pid, 0, &resCount, NULL) >= 0)
+     * updates privateData.
+     * If the process has just been started, we are pretty sure it is not
+     * registered. */
+    if (!(flags & VIR_LOCK_MANAGER_NEW_STARTED) &&
+        sanlock_inquire(-1, priv->vm_pid, 0, &resCount, NULL) >= 0)
         priv->registered = true;
 
     lock->privateData = priv;
