@@ -107,4 +107,12 @@ int qemuProcessReadLog(int fd, char *buf, int buflen, int off, bool skipchar);
 int qemuProcessSetSchedParams(int id, pid_t pid, size_t nsp,
                               virDomainThreadSchedParamPtr sp);
 
+int qemuProcessSPICEAllocatePorts(virQEMUDriverPtr driver,
+                                  virQEMUDriverConfigPtr cfg,
+                                  virDomainGraphicsDefPtr graphics,
+                                  bool allocate);
+
+virDomainDiskDefPtr qemuProcessFindDomainDiskByAlias(virDomainObjPtr vm,
+                                                     const char *alias);
+
 #endif /* __QEMU_PROCESS_H__ */
