@@ -1019,6 +1019,30 @@ virCommandPassListenFDs(virCommandPtr cmd)
     cmd->flags |= VIR_EXEC_LISTEN_FDS;
 }
 
+/*
+ * virCommandPassFDGetFDIndex:
+ * @cmd: pointer to virCommand
+ * @fd: FD to get index of
+ *
+ * Determine the index of the FD in the transfer set.
+ *
+ * Returns index >= 0 if @set contains @fd,
+ * -1 otherwise.
+ */
+int
+virCommandPassFDGetFDIndex(virCommandPtr cmd, int fd)
+{
+    size_t i = 0;
+
+    while (i < cmd->npassfd) {
+        if (cmd->passfd[i].fd == fd)
+            return i;
+        i++;
+    }
+
+    return -1;
+}
+
 /**
  * virCommandSetPidFile:
  * @cmd: the command to modify
