@@ -365,6 +365,38 @@ xenFormatXMDisks(virConfPtr conf, virDomainDefPtr def, int xendConfigVersion)
 }
 
 
+static int
+xenParseXMInputDevs(virConfPtr conf, virDomainDefPtr def)
+{
+    const char *str;
+
+    if (STREQ(def->os.type, "hvm")) {
+        if (xenConfigGetString(conf, "usbdevice", &str, NULL) < 0)
+            return -1;
+        if (str &&
+                (STREQ(str, "tablet") ||
+                 STREQ(str, "mouse") ||
+                 STREQ(str, "keyboard"))) {
+            virDomainInputDefPtr input;
+            if (VIR_ALLOC(input) < 0)
+                return -1;
+
+            input->bus = VIR_DOMAIN_INPUT_BUS_USB;
+            if (STREQ(str, "mouse"))
+                input->type = VIR_DOMAIN_INPUT_TYPE_MOUSE;
+            else if (STREQ(str, "tablet"))
+                input->type = VIR_DOMAIN_INPUT_TYPE_TABLET;
+            else if (STREQ(str, "keyboard"))
+                input->type = VIR_DOMAIN_INPUT_TYPE_KBD;
+            if (VIR_APPEND_ELEMENT(def->inputs, def->ninputs, input) < 0) {
+                virDomainInputDefFree(input);
+                return -1;
+            }
+        }
+    }
+    return 0;
+}
+
 /*
  * Convert an XM config record into a virDomainDef object.
  */
@@ -387,6 +419,9 @@ xenParseXM(virConfPtr conf,
     if (xenParseXMDisk(conf, def, xendConfigVersion) < 0)
          goto cleanup;
 
+    if (xenParseXMInputDevs(conf, def) < 0)
+         goto cleanup;
+
     return def;
 
  cleanup:
@@ -394,13 +429,47 @@ xenParseXM(virConfPtr conf,
     return NULL;
 }
 
+static int
+xenFormatXMInputDevs(virConfPtr conf, virDomainDefPtr def)
+{
+    size_t i;
+    const char *devtype;
+
+    if (STREQ(def->os.type, "hvm")) {
+        for (i = 0; i < def->ninputs; i++) {
+            if (def->inputs[i]->bus == VIR_DOMAIN_INPUT_BUS_USB) {
+                if (xenConfigSetInt(conf, "usb", 1) < 0)
+                    return -1;
+
+                switch (def->inputs[i]->type) {
+                    case VIR_DOMAIN_INPUT_TYPE_MOUSE:
+                        devtype = "mouse";
+                        break;
+                    case VIR_DOMAIN_INPUT_TYPE_TABLET:
+                        devtype = "tablet";
+                        break;
+                    case VIR_DOMAIN_INPUT_TYPE_KBD:
+                        devtype = "keyboard";
+                        break;
+                    default:
+                        continue;
+                }
+                if (xenConfigSetString(conf, "usbdevice", devtype) < 0)
+                    return -1;
+                break;
+            }
+        }
+    }
+    return 0;
+}
+
 
 /* Computing the vcpu_avail bitmask works because MAX_VIRT_CPUS is
    either 32, or 64 on a platform where long is big enough.  */
 verify(MAX_VIRT_CPUS <= sizeof(1UL) * CHAR_BIT);
 
 /*
- * Convert a virDomainDef object inot an XM config record.
+ * Convert a virDomainDef object into an XM config record.
  */
 virConfPtr
 xenFormatXM(virConnectPtr conn,
@@ -418,6 +487,9 @@ xenFormatXM(virConnectPtr conn,
     if (xenFormatXMDisks(conf, def, xendConfigVersion) < 0)
         goto cleanup;
 
+    if (xenFormatXMInputDevs(conf, def) < 0)
+        goto cleanup;
+
     return conf;
 
  cleanup:
