@@ -22,11 +22,30 @@
 
 static virQEMUDriver driver;
 
+enum {
+    WHEN_INACTIVE = 1,
+    WHEN_ACTIVE = 2,
+    WHEN_BOTH = 3,
+};
+
+struct testInfo {
+    char *inName;
+    char *inFile;
+
+    char *outActiveName;
+    char *outActiveFile;
+
+    char *outInactiveName;
+    char *outInactiveFile;
+};
+
 static int
-testCompareXMLToXMLFiles(const char *inxml, const char *outxml, bool live)
+testXML2XMLHelper(const char *inxml,
+                  const char *inXmlData,
+                  const char *outxml,
+                  const char *outXmlData,
+                  bool live)
 {
-    char *inXmlData = NULL;
-    char *outXmlData = NULL;
     char *actual = NULL;
     int ret = -1;
     virDomainDefPtr def = NULL;
@@ -35,11 +54,6 @@ testCompareXMLToXMLFiles(const char *inxml, const char *outxml, bool live)
     if (!live)
         format_flags |= VIR_DOMAIN_DEF_FORMAT_INACTIVE;
 
-    if (virtTestLoadFile(inxml, &inXmlData) < 0)
-        goto fail;
-    if (virtTestLoadFile(outxml, &outXmlData) < 0)
-        goto fail;
-
     if (!(def = virDomainDefParseString(inXmlData, driver.caps, driver.xmlopt,
                                         QEMU_EXPECTED_VIRT_TYPES, parse_flags)))
         goto fail;
@@ -58,89 +72,235 @@ testCompareXMLToXMLFiles(const char *inxml, const char *outxml, bool live)
     }
 
     ret = 0;
+
  fail:
-    VIR_FREE(inXmlData);
-    VIR_FREE(outXmlData);
     VIR_FREE(actual);
     virDomainDefFree(def);
     return ret;
 }
 
-enum {
-    WHEN_INACTIVE = 1,
-    WHEN_ACTIVE = 2,
-    WHEN_EITHER = 3,
-};
-
-struct testInfo {
-    const char *name;
-    bool different;
-    int when;
-};
 
 static int
-testCompareXMLToXMLHelper(const void *data)
+testXML2XMLActive(const void *opaque)
 {
-    const struct testInfo *info = data;
-    char *xml_in = NULL;
-    char *xml_out = NULL;
-    char *xml_out_active = NULL;
-    char *xml_out_inactive = NULL;
+    const struct testInfo *info = opaque;
+
+    return testXML2XMLHelper(info->inName,
+                             info->inFile,
+                             info->outActiveName,
+                             info->outActiveFile,
+                             true);
+}
+
+
+static int
+testXML2XMLInactive(const void *opaque)
+{
+    const struct testInfo *info = opaque;
+
+    return testXML2XMLHelper(info->inName,
+                             info->inFile,
+                             info->outInactiveName,
+                             info->outInactiveFile,
+                             false);
+}
+
+
+static const char testStatusXMLPrefix[] =
+"<domstatus state='running' reason='booted' pid='3803518'>\n"
+"  <taint flag='high-privileges'/>\n"
+"  <monitor path='/var/lib/libvirt/qemu/test.monitor' json='1' type='unix'/>\n"
+"  <vcpus>\n"
+"    <vcpu pid='3803519'/>\n"
+"  </vcpus>\n"
+"  <qemuCaps>\n"
+"    <flag name='vnc-colon'/>\n"
+"    <flag name='no-reboot'/>\n"
+"    <flag name='drive'/>\n"
+"    <flag name='name'/>\n"
+"    <flag name='uuid'/>\n"
+"    <flag name='vnet-hdr'/>\n"
+"    <flag name='qxl.vgamem_mb'/>\n"
+"    <flag name='qxl-vga.vgamem_mb'/>\n"
+"    <flag name='pc-dimm'/>\n"
+"  </qemuCaps>\n"
+"  <devices>\n"
+"    <device alias='balloon0'/>\n"
+"    <device alias='video0'/>\n"
+"    <device alias='serial0'/>\n"
+"    <device alias='net0'/>\n"
+"    <device alias='usb'/>\n"
+"  </devices>\n";
+
+static const char testStatusXMLSuffix[] =
+"</domstatus>\n";
+
+
+static int
+testCompareStatusXMLToXMLFiles(const void *opaque)
+{
+    const struct testInfo *data = opaque;
+    virBuffer buf = VIR_BUFFER_INITIALIZER;
+    xmlDocPtr xml = NULL;
+    virDomainObjPtr obj = NULL;
+    char *expect = NULL;
+    char *actual = NULL;
+    char *source = NULL;
     int ret = -1;
+    int keepBlanksDefault = xmlKeepBlanksDefault(0);
 
-    if (virAsprintf(&xml_in, "%s/qemuxml2argvdata/qemuxml2argv-%s.xml",
-                    abs_srcdir, info->name) < 0 ||
-        virAsprintf(&xml_out, "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s.xml",
-                    abs_srcdir, info->name) < 0 ||
-        virAsprintf(&xml_out_active,
-                    "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s-active.xml",
-                    abs_srcdir, info->name) < 0 ||
-        virAsprintf(&xml_out_inactive,
-                    "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s-inactive.xml",
-                    abs_srcdir, info->name) < 0)
+    /* construct faked source status XML */
+    virBufferAdd(&buf, testStatusXMLPrefix, -1);
+    virBufferAdjustIndent(&buf, 2);
+    virBufferAddStr(&buf, data->inFile);
+    virBufferAdjustIndent(&buf, -2);
+    virBufferAdd(&buf, testStatusXMLSuffix, -1);
+
+    if (!(source = virBufferContentAndReset(&buf))) {
+        fprintf(stderr, "Failed to create the source XML");
         goto cleanup;
-
-    if ((info->when & WHEN_INACTIVE)) {
-        char *out;
-        if (!info->different)
-            out = xml_in;
-        else if (virFileExists(xml_out_inactive))
-            out = xml_out_inactive;
-        else
-            out = xml_out;
-
-        if (testCompareXMLToXMLFiles(xml_in, out, false) < 0)
-            goto cleanup;
     }
 
-    if ((info->when & WHEN_ACTIVE)) {
-        char *out;
-        if (!info->different)
-            out = xml_in;
-        else if (virFileExists(xml_out_active))
-            out = xml_out_active;
-        else
-            out = xml_out;
+    /* construct the expect string */
+    virBufferAdd(&buf, testStatusXMLPrefix, -1);
+    virBufferAdjustIndent(&buf, 2);
+    virBufferAddStr(&buf, data->outActiveFile);
+    virBufferAdjustIndent(&buf, -2);
+    virBufferAdd(&buf, testStatusXMLSuffix, -1);
 
-        if (testCompareXMLToXMLFiles(xml_in, out, true) < 0)
-            goto cleanup;
+    if (!(expect = virBufferContentAndReset(&buf))) {
+        fprintf(stderr, "Failed to create the expect XML");
+        goto cleanup;
+    }
+
+    /* parse the fake source status XML */
+    if (!(xml = virXMLParseString(source, "(domain_status_test_XML)")) ||
+        !(obj = virDomainObjParseNode(xml, xmlDocGetRootElement(xml),
+                                      driver.caps, driver.xmlopt,
+                                      QEMU_EXPECTED_VIRT_TYPES,
+                                      VIR_DOMAIN_DEF_PARSE_STATUS |
+                                      VIR_DOMAIN_DEF_PARSE_ACTUAL_NET |
+                                      VIR_DOMAIN_DEF_PARSE_PCI_ORIG_STATES |
+                                      VIR_DOMAIN_DEF_PARSE_CLOCK_ADJUST))) {
+        fprintf(stderr, "Failed to parse domain status XML:\n%s", source);
+        goto cleanup;
+    }
+
+    /* format it back */
+    if (!(actual = virDomainObjFormat(driver.xmlopt, obj,
+                                      VIR_DOMAIN_DEF_FORMAT_SECURE))) {
+        fprintf(stderr, "Failed to format domain status XML");
+        goto cleanup;
+    }
+
+    if (STRNEQ(actual, expect)) {
+        virtTestDifferenceFull(stderr,
+                               expect, data->outActiveName,
+                               actual, data->inName);
+        goto cleanup;
     }
 
     ret = 0;
 
  cleanup:
-    VIR_FREE(xml_in);
-    VIR_FREE(xml_out);
-    VIR_FREE(xml_out_active);
-    VIR_FREE(xml_out_inactive);
+    xmlKeepBlanksDefault(keepBlanksDefault);
+    xmlFreeDoc(xml);
+    virObjectUnref(obj);
+    VIR_FREE(expect);
+    VIR_FREE(actual);
+    VIR_FREE(source);
     return ret;
 }
 
 
+static void
+testInfoFree(struct testInfo *info)
+{
+    VIR_FREE(info->inName);
+    VIR_FREE(info->inFile);
+
+    VIR_FREE(info->outActiveName);
+    VIR_FREE(info->outActiveFile);
+
+    VIR_FREE(info->outInactiveName);
+    VIR_FREE(info->outInactiveFile);
+}
+
+
+static int
+testInfoSet(struct testInfo *info,
+            const char *name,
+            bool different,
+            int when)
+{
+    if (virAsprintf(&info->inName, "%s/qemuxml2argvdata/qemuxml2argv-%s.xml",
+                    abs_srcdir, name) < 0)
+        goto error;
+
+    if (virtTestLoadFile(info->inName, &info->inFile) < 0)
+        goto error;
+
+    if (when & WHEN_INACTIVE) {
+        if (different) {
+            if (virAsprintf(&info->outInactiveName,
+                           "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s-inactive.xml",
+                           abs_srcdir, name) < 0)
+                goto error;
+
+            if (!virFileExists(info->outInactiveName)) {
+                VIR_FREE(info->outInactiveName);
+
+                if (virAsprintf(&info->outInactiveName,
+                                "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s.xml",
+                                abs_srcdir, name) < 0)
+                    goto error;
+            }
+        } else {
+            if (VIR_STRDUP(info->outInactiveName, info->inName) < 0)
+                goto error;
+        }
+
+        if (virtTestLoadFile(info->outInactiveName, &info->outInactiveFile) < 0)
+            goto error;
+    }
+
+    if (when & WHEN_ACTIVE) {
+        if (different) {
+            if (virAsprintf(&info->outActiveName,
+                           "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s-active.xml",
+                           abs_srcdir, name) < 0)
+                goto error;
+
+            if (!virFileExists(info->outActiveName)) {
+                VIR_FREE(info->outActiveName);
+
+                if (virAsprintf(&info->outActiveName,
+                                "%s/qemuxml2xmloutdata/qemuxml2xmlout-%s.xml",
+                                abs_srcdir, name) < 0)
+                    goto error;
+            }
+        } else {
+            if (VIR_STRDUP(info->outActiveName, info->inName) < 0)
+                goto error;
+        }
+
+        if (virtTestLoadFile(info->outActiveName, &info->outActiveFile) < 0)
+            goto error;
+    }
+
+    return 0;
+
+ error:
+    testInfoFree(info);
+    return -1;
+}
+
+
 static int
 mymain(void)
 {
     int ret = 0;
+    struct testInfo info;
 
     if ((driver.caps = testQemuCapsInit()) == NULL)
         return EXIT_FAILURE;
@@ -148,19 +308,36 @@ mymain(void)
     if (!(driver.xmlopt = virQEMUDriverCreateXMLConf(&driver)))
         return EXIT_FAILURE;
 
-# define DO_TEST_FULL(name, is_different, when)                         \
-    do {                                                                \
-        const struct testInfo info = {name, is_different, when};        \
-        if (virtTestRun("QEMU XML-2-XML " name,                         \
-                        testCompareXMLToXMLHelper, &info) < 0)          \
-            ret = -1;                                                   \
+# define DO_TEST_FULL(name, is_different, when)                                \
+    do {                                                                       \
+        if (testInfoSet(&info, name, is_different, when) < 0) {                \
+            fprintf(stderr, "Failed to generate test data for '%s'", name);    \
+            return -1;                                                         \
+        }                                                                      \
+                                                                               \
+        if (info.outInactiveName) {                                            \
+            if (virtTestRun("QEMU XML-2-XML-inactive " name,                   \
+                            testXML2XMLInactive, &info) < 0)                   \
+                ret = -1;                                                      \
+        }                                                                      \
+                                                                               \
+        if (info.outActiveName) {                                              \
+            if (virtTestRun("QEMU XML-2-XML-active " name,                     \
+                            testXML2XMLActive, &info) < 0)                     \
+                ret = -1;                                                      \
+                                                                               \
+            if (virtTestRun("QEMU XML-2-XML-status " name,                     \
+                            testCompareStatusXMLToXMLFiles, &info) < 0)        \
+                ret = -1;                                                      \
+        }                                                                      \
+        testInfoFree(&info);                                                   \
     } while (0)
 
 # define DO_TEST(name) \
-    DO_TEST_FULL(name, false, WHEN_EITHER)
+    DO_TEST_FULL(name, false, WHEN_BOTH)
 
 # define DO_TEST_DIFFERENT(name) \
-    DO_TEST_FULL(name, true, WHEN_EITHER)
+    DO_TEST_FULL(name, true, WHEN_BOTH)
 
     /* Unset or set all envvars here that are copied in qemudBuildCommandLine
      * using ADD_ENV_COPY, otherwise these tests may fail due to unexpected
@@ -178,7 +355,6 @@ mymain(void)
     DO_TEST("boot-menu-disable");
     DO_TEST_DIFFERENT("boot-menu-disable-with-timeout");
     DO_TEST("boot-order");
-    DO_TEST("bootloader");
 
     DO_TEST("reboot-timeout-enabled");
     DO_TEST("reboot-timeout-disabled");
@@ -259,7 +435,6 @@ mymain(void)
     DO_TEST("graphics-spice-qxl-vga");
     DO_TEST("input-usbmouse");
     DO_TEST("input-usbtablet");
-    DO_TEST("input-xen");
     DO_TEST("misc-acpi");
     DO_TEST("misc-disable-s3");
     DO_TEST("misc-disable-suspends");
@@ -275,6 +450,7 @@ mymain(void)
     DO_TEST("net-virtio-network-portgroup");
     DO_TEST("net-hostdev");
     DO_TEST("net-hostdev-vfio");
+    DO_TEST("net-midonet");
     DO_TEST("net-openvswitch");
     DO_TEST("sound");
     DO_TEST("sound-device");
@@ -356,6 +532,7 @@ mymain(void)
 
     /* These tests generate different XML */
     DO_TEST_DIFFERENT("balloon-device-auto");
+    DO_TEST_DIFFERENT("balloon-device-period");
     DO_TEST_DIFFERENT("channel-virtio-auto");
     DO_TEST_DIFFERENT("console-compat-auto");
     DO_TEST_DIFFERENT("disk-scsi-device-auto");
@@ -369,6 +546,7 @@ mymain(void)
     DO_TEST_DIFFERENT("usb-ich9-ehci-addr");
 
     DO_TEST_DIFFERENT("metadata");
+    DO_TEST_DIFFERENT("metadata-duplicate");
 
     DO_TEST("tpm-passthrough");
     DO_TEST("pci-bridge");
@@ -402,6 +580,7 @@ mymain(void)
     DO_TEST("pcihole64-q35");
 
     DO_TEST("panic");
+    DO_TEST("panic-no-address");
 
     DO_TEST_DIFFERENT("disk-backing-chains");
 
@@ -409,6 +588,7 @@ mymain(void)
 
     DO_TEST_DIFFERENT("cpu-numa1");
     DO_TEST_DIFFERENT("cpu-numa2");
+    DO_TEST_DIFFERENT("cpu-numa-no-memory-element");
     DO_TEST("cpu-numa-disjoint");
     DO_TEST("cpu-numa-memshared");
 
@@ -417,11 +597,17 @@ mymain(void)
     DO_TEST("numatune-memnode-no-memory");
 
     DO_TEST("bios-nvram");
+    DO_TEST_DIFFERENT("bios-nvram-os-interleave");
 
     DO_TEST("tap-vhost");
     DO_TEST_DIFFERENT("tap-vhost-incorrect");
     DO_TEST("shmem");
     DO_TEST("smbios");
+    DO_TEST("aarch64-aavmf-virtio-mmio");
+
+    DO_TEST("memory-hotplug");
+    DO_TEST("memory-hotplug-nonuma");
+    DO_TEST("memory-hotplug-dimm");
 
     virObjectUnref(driver.caps);
     virObjectUnref(driver.xmlopt);
