@@ -1,7 +1,7 @@
 /*
  * virsh-interface.c: Commands to manage host interface
  *
- * Copyright (C) 2005, 2007-2014 Red Hat, Inc.
+ * Copyright (C) 2005, 2007-2015 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -507,10 +507,11 @@ cmdInterfaceDumpXML(vshControl *ctl, const vshCmd *cmd)
  */
 static const vshCmdInfo info_interface_define[] = {
     {.name = "help",
-     .data = N_("define (but don't start) a physical host interface from an XML file")
+     .data = N_("define an inactive persistent physical host interface or "
+                "modify an existing persistent one from an XML file")
     },
     {.name = "desc",
-     .data = N_("Define a physical host interface.")
+     .data = N_("Define or modify a persistent physical host interface.")
     },
     {.name = NULL}
 };
